@@ -39,7 +39,7 @@
 // ***** END LICENSE BLOCK *****
 
 var EXPORTED_SYMBOLS = ["MozMillController", "waitForEval", "MozMillAsyncTest",
-                        "globalEventRegistry", "sleep"];
+                        "globalEventRegistry", "sleep", "windowMap"];
 
 var EventUtils = {}; Components.utils.import('resource://mozmill/stdlib/EventUtils.js', EventUtils);
 
@@ -54,6 +54,42 @@ var hwindow = Components.classes["@mozilla.org/appshell/appShellService;1"]
 var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].
      getService(Components.interfaces.nsIConsoleService);
 
+// The window map which is used to store information e.g. loaded state of each
+// open chrome and content window.
+var windowMap = {
+  _windows : { },
+
+  contains : function (aWindowId) {
+    return (aWindowId in this._windows);
+  },
+
+  getValue : function (aWindowId, aProperty) {
+    if (!this.contains(aWindowId)) {
+      return undefined;
+    } else {
+      var win = this._windows[aWindowId];
+
+      return (aProperty in win) ? win[aProperty]
+                                : undefined;
+    }
+  },
+
+  remove : function (aWindowId) {
+    if (this.contains(aWindowId))
+      delete this._windows[aWindowId];
+    // dump("* current map: " + JSON.stringify(this._windows) + "\n");
+  },
+
+  update : function (aWindowId, aProperty, aValue) {
+    if (!this.contains(aWindowId))
+      this._windows[aWindowId] = { };
+
+    this._windows[aWindowId][aProperty] = aValue;
+    // dump("* current map: " + JSON.stringify(this._windows) + "\n");
+  }
+}
+
+
 // Declare most used utils functions in the controller namespace
 var sleep = utils.sleep;
 var assert = utils.assert;
@@ -308,10 +344,12 @@ var MozMillController = function (window) {
   Components.utils.import('resource://mozmill/modules/mozmill.js', this.mozmillModule);
 
   utils.waitFor(function() {
-    return window != null &&
-           ("documentLoaded" in window) &&
-           window.documentLoaded;
-  }, "controller(): Window could not be initialized.");
+    return window != null && this.isLoaded();
+  }, "controller(): Window could not be initialized.", undefined, undefined, this);
+
+  // Ensure to focus the window. It will move it virtually into the foreground
+  // when focusmanager.testmode is enabled
+  this.window.focus();
 
   if ( controllerAdditions[window.document.documentElement.getAttribute('windowtype')] != undefined ) {
     this.prototype = new utils.Copy(this.prototype);
@@ -634,6 +672,18 @@ MozMillController.prototype.radio = function(el)
   return true;
 }
 
+/**
+ * Checks if the specified window has been loaded
+ *
+ * @param {DOMWindow} [window=this.window] Window object to check for loaded state
+ */
+MozMillController.prototype.isLoaded = function (window) {
+  var win = window || this.window;
+
+  var id = utils.getWindowId(win);
+  return windowMap.contains(id) && windowMap.getValue(id, "loaded");
+}
+
 MozMillController.prototype.waitFor = function(callback, message, timeout,
                                                interval, thisObject) {
   utils.waitFor(callback, message, timeout, interval, thisObject);
@@ -1257,32 +1307,29 @@ Tabs.prototype.selectTabIndex = function(i) {
 function browserAdditions (controller) {
   controller.tabs = new Tabs(controller);
 
-  controller.waitForPageLoad = function(aTabDocument, aTimeout, aInterval) {
+  controller.waitForPageLoad = function(aDocument, aTimeout, aInterval) {
     var timeout = aTimeout || 30000;
-    var tab = null;
+    var win = null;
 
     // If a user tries to do waitForPageLoad(2000), this will assign the
     // interval the first arg which is most likely what they were expecting
-    if (typeof(aTabDocument) == "number"){
-      timeout = aTabDocument;
-    }
+    if (typeof(aDocument) === "number")
+      timeout = aDocument;
 
-    // Find the browser element for the given aTabDocument
-    if (aTabDocument && typeof(aTabDocument) == "object") {
-      tab = this.window.gBrowser.getBrowserForDocument(aTabDocument);
+    // If we have a real document use its default view
+    if (aDocument && (typeof(aDocument) === "object") &&
+        "defaultView" in aDocument)
+      win = aDocument.defaultView;
 
-      if (!tab) {
-        throw new Error("controller.waitForPageLoad(): Specified tab hasn't been found.");
-      }
-    }
-
-    // If no tab has been specified, fallback to the selected browser
-    tab = tab || this.window.gBrowser.selectedBrowser;
+    // If no document has been specified, fallback to the default view of the
+    // currently selected tab browser
+    win = win || this.window.gBrowser.selectedBrowser.contentWindow;
 
     // Wait until the content in the tab has been loaded
-    this.waitFor(function() {
-      return ("documentLoaded" in tab && tab.documentLoaded);
-    }, "controller.waitForPageLoad(): Timeout waiting for page loaded.", timeout, aInterval);
+    this.waitFor(function () {
+      return this.isLoaded(win);
+    }, "controller.waitForPageLoad(): Timeout waiting for page loaded.",
+       timeout, aInterval, this);
 
     frame.events.pass({'function':'controller.waitForPageLoad()'});
   }
